/*****************************************************************************************/
/* 
* Milliwattmeter mit 2 Messbereichen (2mW und 2 W) für 24 GHz-Detektor.
* 
* Anzeige auf OLED-Display 128x64 und zusätzlich Analog-Anzeigeinstrument (100µA Vollausschlag)
* optionale Ausgabe auf seriellem Port RS232 mit 9600 baud
* V 1.0: Realisierung mit Messbereich 2mW und 2000mW, Auswahl an Pin2, Analoganzeige auf Pin10 
* 
* 
* 
* 
* Autor: Wolfgang Schneider, DJ8ES, 16.07.2018
*
* Hardware: Arduino Uno –  RF-Detektor 24 GHz ADC: ADC0 (2mW), ADC1 (2000mW) 
*           5V          –  Vcc  
*           GND         –  GND
*           A4 / SDA    –  SDA (OLED-Display) 
*           A5 / SCL    –  SCL (OLED-Display) 
*           D10         -  PWM-Ausgang für Analoganzeige 
*           D2          -  D2: 0 (2mW), 1 (2000mW) 
*/
/*****************************************************************************************/

//#include <Wire.h> 
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4 
Adafruit_SSD1306 display(OLED_RESET);

/*** Software release and date ***/
const char* sketchname =  "mW-Meter 24GHz";
const char* revision   =  "V1.0";
const char* author     =  "Wolfgang Schneider";
const char* call       =  "DJ8ES";
const char* date       =  "Jul 2018";

/*** Declare constants and variables for Serial Interface ***/
const unsigned int BAUDRATE = 9600;   // Serial monitor baudrate

/*** Declare constants and variables ***/
unsigned long analogIn;
unsigned int analogOut;
unsigned int range;
int analogAnz = 0;
float mW = 0.0;
float dBm = 0.0;
float Umess = 0.0;    // Umess in mV

/*** Declare Output PWM on Pin 10 ***/
int analogPin = 10;

/*****************************************************************************************/


/*****************************************************************************************/
void setup() {
  pinMode(analogPin,OUTPUT);  
  pinMode(2, INPUT); digitalWrite(2,1);      // Auswahl Messbreich , digitalWrite() setzt internen PullUp-Widerstand

  // initialize with the I2C addr 0x3C 
  // display.begin(SSD1306_SWITCHCAPVCC, 0x3D);   // für 128X32 
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);      // für 128X64
  
  // Ansteuerung OLED 128x64 mit 0,96"
  display.clearDisplay();
  
  display.setTextColor(WHITE);        // Textfarbe setzen
  display.setTextSize(1);             // Textgroesse setzen

  display.setCursor(1,10);            // Textstartposition einstellen
  display.print(sketchname);          // Text anzeigen
  display.setCursor(96,10);           
  display.print(revision);            

  display.setCursor(1,30);            
  display.print(call);                
  display.setCursor(72,30);           
  display.print(date);                
  display.setCursor(1,40);            
  display.print(author);              

  display.display();
  
  delay(3000);                        // Wait 3s
  display.clearDisplay();
  
  display.display();


  // Ansteuerung der seriellen Schnittstelle (optional)
  //Serial.begin(BAUDRATE);             // Initialze serial monitor

  //Serial.println();
  //Serial.print(sketchname); Serial.print("\t"); Serial.print("\t"); Serial.println(revision);
  //Serial.print(author); Serial.print("\t");   Serial.println(date);
  //Serial.println();

} // End of void setup()
/*****************************************************************************************/


/*****************************************************************************************/
void loop() {
  // Variablen zurück setzen
  mW = 0.0;
  dBm = 0.0;

  //Auswahl Messbereich
  //range=1;                                              //Messbereich 1 (2000mW) setzen (nur Test)
  if (digitalRead(2)==0){
         range=0;                                     //Messbereich 0 (2mW) setzen
     }
     else {
         range=1;                                     //Messbereich 1 (2000mW) setzen (Default)
     }

  analogIn = 0;
  if (range==0)          // Bereich 2mW
     {
        for(int i=0; i<128; i++)
        {
            // Einlesen der Messwerte an Pin A0 bzw. A1, (jeweils 128 Messungen mit anschließender Mittelung)
            analogIn = analogIn + analogRead(0);        //Lese Analogport A1
        }
     }
     else                // Bereich 2000mW
     {
        for(int i=0; i<128; i++)
        {
            // Einlesen der Messwerte an Pin A0 bzw. A1, (jeweils 128 Messungen mit anschließender Mittelung)
            analogIn = analogIn + analogRead(1);        //Lese Analogport A2
        }
     }
  analogIn = analogIn / 128;                    //Division wg. 128 Messungen
  analogIn = map(analogIn,0,1023,0,5000);       //Normierung auf 0..5000mV
  Umess = analogIn;

  // Berechnung der Leistung mit Polynom 2. Ordnung 
  // Bsp 2. Ordnung: W = 0.01004*Umess*Umess + 0.02324*Umess - 0.04017;
  if (range==0)          // Bereich 2mW
     {
         mW = 1.15e-6*Umess*Umess + 0*Umess + 0;
     }
     else                // Bereich 2000mW
     {
         mW = 0.16e-3*Umess*Umess + 0*Umess + 0;
     }

  dBm = 10 * log10(mW);             // Berechnung dBm


  // Vollausschlag für Anzeigeinstrument abhängig vom Messbereich setzen
  switch (range) {
    case 0:
       analogAnz = int(1000*mW);
       analogOut = map(analogAnz,0,2000,0,255);    //Normierung auf Vollausschlag (PWM=255) für 2mW ... 255
       break;
    case 1:
       analogAnz = int(mW);
       analogOut = map(analogAnz,0,2000,0,255);    //Normierung auf Vollausschlag (PWM=255) für für 2000mW ... 255
       break;
  }


  analogWrite(analogPin,analogOut);     // Anzeigeinstrument mit PWM (255 = Vollausschlag)
  DisplayOut();                         // Ausgabe der Messwerte auf OLED-Display 128x64 mit 0,96"
  //PowerSerialOut();                     // Ausgabe der Messwerte auf seriellem Interface (optional)

  //Serial.print(digitalRead(7));Serial.print(digitalRead(6));Serial.print(digitalRead(5));Serial.print(digitalRead(4));Serial.print(digitalRead(3));Serial.println(digitalRead(2));
  //Serial.println(range);

  delay(100);         // ca. 10 Messungen pro Sekunde

} // End of void loop()
/*****************************************************************************************/


/*****************************************************************************************/
// Unterprogramme und Funktionen

void DisplayOut() {
  display.clearDisplay();

  display.setTextSize(1);                       // Zeichengroesse setzen
  display.setCursor(1,0);                       // Textstartposition einstellen
  //display.println("QRP-Wattmeter  DJ8ES");      // Text anzeigen
  display.println("Powermeter     24GHz");      // Text anzeigen

  display.setTextSize(2);                       
  display.setCursor(1,20);                      
  display.print("P: ");                         
  switch (range) {
    case 0:
       display.print(mW,3);                 // 3 Nachkommastellen anzeigen
       break;
    case 1:
       display.print(mW,1);                 // 1 Nachkommastelle anzeigen
       break;
  }

  // Anzeige in mW bzw. dBm 
  display.setTextSize(1);
  display.print(" mW");                          

  display.setTextSize(2);                       
  display.setCursor(1,45);                      
  display.print("P: ");                         
  display.print(dBm,1);                     // 1 Nachkommastelle anzeigen
  display.setTextSize(1);                       
  display.print(" dBm");                        

  display.display();
}


void PowerSerialOut() {
  //Serial.println(analogOut);
  //Serial.println();
}

/*****************************************************************************************/

